@@ -39,9 +39,13 @@ ggplot_heatmap <- function(xx,
                            row_text_angle = 0,
                            column_text_angle = 45,
                            scale_fill_gradient_fun =
-                             scale_fill_gradientn(colors = viridis(n=256, alpha = 1, begin = 0,
-                                                                   end = 1, option = "viridis"),
-                                                  na.value = "grey50", limits = NULL),
+                           scale_fill_gradientn(
+                             colors = viridis(
+                               n = 256, alpha = 1, begin = 0,
+                               end = 1, option = "viridis"
+                             ),
+                             na.value = "grey50", limits = NULL
+                           ),
                            grid_color = NA,
                            grid_size = 0.1,
                            key.title = NULL,
@@ -50,17 +54,27 @@ ggplot_heatmap <- function(xx,
                            label_names = NULL,
                            fontsize_row = 10,
                            fontsize_col = 10,
+                           type = c("heatmap", "scatter"),
+                           pointsize = 5,
+                           point_size_mat = NULL,
+                           label_format_fun = function(...) format(..., digits = 4),
+                           point_size_name = "Point size",
+                           custom_hovertext = NULL,
                            ...) {
-  theme_clear_grid_heatmap <- theme(axis.line = element_line(color = "black"),
-                                    panel.grid.major = element_blank(),
-                                    panel.grid.minor = element_blank(),
-                                    panel.border = element_blank(),
-                                    panel.background = element_blank())
+  theme_clear_grid_heatmap <- theme(
+    axis.line = element_line(color = "black"),
+    panel.grid.major = element_blank(),
+    panel.grid.minor = element_blank(),
+    panel.border = element_blank(),
+    panel.background = element_blank()
+  )
+
+  type <- match.arg(type)
   # heatmap
   # xx <- x$matrix$data
 
   df <- xx
-  if(!is.data.frame(df)) df <- as.data.frame(df, check.rows = FALSE)
+  if (!is.data.frame(df)) df <- as.data.frame(df, check.rows = FALSE)
 
   if (is.null(label_names)) {
     if (is.null(dim_names <- names(dimnames(df)))) {
@@ -71,15 +85,124 @@ ggplot_heatmap <- function(xx,
   } else {
     assert_that(length(label_names) == 3)
   }
+
+  mdf <- melt_df(xx, label_names)
+  if (!is.null(point_size_mat)) {
+    ps_label_names <- label_names
+    ps_label_names[[3]] <- point_size_name
+    point_size_mat <- melt_df(point_size_mat, ps_label_names)
+    mdf <- cbind(mdf, point_size_mat[point_size_name])
+  }
+  row <- label_names[[1]]
+  col <- label_names[[2]]
+  val <- label_names[[3]]
+
+  mdf[["text"]] <- paste0(
+    row, ": ", mdf[[1]], "<br>",
+    col, ": ", mdf[[2]], "<br>",
+    val, ": ", label_format_fun(mdf[[3]])
+  )
+
+  if (type == "heatmap") {
+    geom <- "geom_tile"
+    aes_args <- list(fill = paste_aes(val))
+    geom_args <- list(
+      # mapping = aes_string(fill = paste_aes(val)),
+      color = grid_color,
+      size = grid_size
+    )
+  } else if (type == "scatter") {
+    geom <- "geom_point"
+    geom_args <- list()
+    if (!is.null(point_size_mat)) {
+      mdf[["text"]] <- paste(
+        mdf[["text"]], "<br>",
+        point_size_name, ": ", label_format_fun(mdf[[4]])
+      )
+      aes_args <- list(color = paste_aes(val), text = "text", size = paste_aes(point_size_name))
+
+      # geom_args[["mapping"]] <- aes_string(
+      #   color = paste_aes(val),
+      #   text = "text",
+      #   size = paste_aes(point_size_name)
+      # )
+    } else {
+      geom_args[["size"]] <- grid_size
+      aes_args <- list(color = paste_aes(val), text = "text")
+      # geom_args[["mapping"]] <- aes_string(color = paste_aes(val), text = "text")
+    }
+  }
+  if (!is.null(custom_hovertext)) {
+    mdf[["text"]] <- paste0(mdf[["text"]], "<br>", custom_hovertext)
+    aes_args[["text"]] <- "text"
+  }
+  geom_args[["mapping"]] <- do.call(aes_string, aes_args)
+
+
+  if (!is.null(custom_hovertext)) {
+    geom_args[["text"]] <- "text"
+  }
+  # TODO:
+  # http://stackoverflow.com/questions/15921799/draw-lines-around-specific-areas-in-geom-tile
+  # https://cran.r-project.org/web/packages/viridis/vignettes/intro-to-viridis.html
+  p <- ggplot(mdf, aes_string(x = col, y = row)) +
+    ## Using the text aes produces a warning... Not ideal!
+    suppressWarnings(do.call(geom, geom_args)) +
+    scale_fill_gradient_fun +
+    theme_bw() +
+    theme_clear_grid_heatmap +
+    theme(
+      axis.text.x = element_text(
+        angle = column_text_angle,
+        size = fontsize_col, hjust = 1
+      ),
+      axis.text.y = element_text(
+        angle = row_text_angle,
+        size = fontsize_row, hjust = 1
+      )
+    )
+
+  if (type == "scatter") {
+    p <- p +
+      coord_cartesian(xlim = c(1, ncol(xx)), ylim = c(1, nrow(xx)))
+  } else {
+    p <- p + coord_cartesian(expand = FALSE)
+  }
+  ## Passed in to allow users to alter (courtesy of GenVisR)
+  if (!missing(layers)) {
+    p <- p + layers
+  }
+
+  # p <- p + scale_x_discrete(limits = unique(mdf))
+  # http://stats.stackexchange.com/questions/5007/how-can-i-change-the-title-of-a-legend-in-ggplot2
+  p <- p + labs(fill = key.title)
+
+  # until this bug is fixed: https://github.com/ropensci/plotly/issues/699
+  # we are forced to use geom_hline and geom_vline
+  if (!is.na(grid_color)) {
+    p <- p + geom_hline(yintercept = c(0:nrow(xx)) + 0.5, color = grid_color) # , size = grid_size # not implemented since it doesn't work with plotly
+    p <- p + geom_vline(xintercept = c(0:ncol(xx)) + 0.5, color = grid_color) # , size = grid_size # not implemented since it doesn't work with plotly
+  }
+
+  if (row_dend_left) p <- p + scale_y_discrete(position = "right") # possible as of ggplot 2.1.0 !
+
+  p
+}
+
+melt_df <- function(x, label_names) {
+  # heatmap
+  # xx <- x$matrix$data
+  if (!is.data.frame(x)) df <- as.data.frame(x)
+
   row <- label_names[[1]]
   col <- label_names[[2]]
   val <- label_names[[3]]
 
   # colnames(df) <- x$matrix$cols
-  if(!is.null(rownames(df))) {
-    df[[row]] <- make.unique(rownames(df), "_")
+  if (!is.null(rownames(x))) {
+    df[[row]] <- rownames(x)
   } else {
-    df[[row]] <- 1:nrow(df)
+    df[[row]] <- 1:nrow(x)
   }
 
   df[[row]] <- factor(
@@ -88,55 +211,23 @@ ggplot_heatmap <- function(xx,
     ordered = TRUE
   )
 
-
   mdf <- reshape2::melt(df, id.vars = row)
   colnames(mdf)[2:3] <- c(col, val) # rename "variable" and "value"
-
-  # TODO:
-  # http://stackoverflow.com/questions/15921799/draw-lines-around-specific-areas-in-geom-tile
-  # https://cran.r-project.org/web/packages/viridis/vignettes/intro-to-viridis.html
-  p <- ggplot(mdf, aes_string(x = col, y = row)) +
-    geom_tile(aes_string(fill = val), color = grid_color, size = grid_size) +
-    # scale_linetype_identity() +
-    # scale_fill_viridis() +
-    coord_cartesian(expand = FALSE) +
-    scale_fill_gradient_fun +
-    theme_bw() + theme_clear_grid_heatmap +
-    theme(axis.text.x = element_text(angle = column_text_angle,
-            size = fontsize_col, hjust = 1),
-          axis.text.y = element_text(angle = row_text_angle,
-            size = fontsize_row, hjust = 1)
-          )
-
-  if(!missing(layers)) p <- p + layers
-    ## Passed in to allow users to alter (courtesy of GenVisR)
-
-  # p <- p + scale_x_discrete(limits = unique(mdf))
-  # http://stats.stackexchange.com/questions/5007/how-can-i-change-the-title-of-a-legend-in-ggplot2
-  p <- p + labs(fill = key.title)
-
-  # until this bug is fixed: https://github.com/ropensci/plotly/issues/699
-  # we are forced to use geom_hline and geom_vline
-  if(!is.na(grid_color)) {
-    p <- p + geom_hline(yintercept = c(0:nrow(xx)) + 0.5, color = grid_color) # , size = grid_size # not implemented since it doesn't work with plotly
-    p <- p + geom_vline(xintercept = c(0:ncol(xx)) + 0.5, color = grid_color) # , size = grid_size # not implemented since it doesn't work with plotly
-
-  }
-
-  if(row_dend_left) p <- p + scale_y_discrete(position = "right") # possible as of ggplot 2.1.0 !
-
-  p
+  mdf
 }
 
+paste_aes <- function(x) {
+  paste0("`", x, "`")
+}
 
 plotly_heatmap <- function(x, limits = range(x),
-    colors = viridis(n=256, alpha = 1, begin = 0, end = 1, option = "viridis"),
-    row_text_angle = 0, column_text_angle = 45, grid.color, grid.size,
-    row_dend_left = FALSE, fontsize_row = 10, fontsize_col = 10, key_title = "",
-    colorbar_xanchor = "left", colorbar_yanchor = "bottom",
-    label_names = NULL,
-    colorbar_xpos = 1.1, colorbar_ypos = 1, colorbar_len = 0.3) {
-
+                           colors = viridis(n = 256, alpha = 1, begin = 0, end = 1, option = "viridis"),
+                           row_text_angle = 0, column_text_angle = 45, grid.color, grid.size,
+                           row_dend_left = FALSE, fontsize_row = 10, fontsize_col = 10, key_title = "",
+                           colorbar_xanchor = "left", colorbar_yanchor = "bottom",
+                           label_names = NULL,
+                           colorbar_xpos = 1.1, colorbar_ypos = 1, colorbar_len = 0.3,
+                           custom_hovertext = NULL) {
   if (is.function(colors)) colors <- colors(256)
 
 
@@ -151,7 +242,8 @@ plotly_heatmap <- function(x, limits = range(x),
   }
 
   text_mat <- as.data.frame(x)
-  text_mat[] <- lapply(seq_along(text_mat),
+  text_mat[] <- lapply(
+    seq_along(text_mat),
     function(i) {
       paste0(
         label_names[3], ": ", x[, i], "<br>",
@@ -161,32 +253,38 @@ plotly_heatmap <- function(x, limits = range(x),
     }
   )
   text_mat <- as.matrix(text_mat)
+  if (!is.null(custom_hovertext)) {
+    text_mat <- paste(text_mat, custom_hovertext, sep = "<br>")
+  }
 
-
-  p <- plot_ly(z = x, x = 1:ncol(x), y = 1:nrow(x), text = text_mat,
+  p <- plot_ly(
+    z = x, x = 1:ncol(x), y = 1:nrow(x), text = text_mat,
     type = "heatmap", showlegend = FALSE, colors = colors, hoverinfo = "text",
-    zmin = limits[1], zmax = limits[2]) %>%
-      layout(
-        xaxis = list(
-          tickfont = list(size = fontsize_col),
-          tickangle = column_text_angle,
-          tickvals = 1:ncol(x), ticktext = colnames(x),
-          linecolor = "#ffffff",
-          range = c(0.5, ncol(x) + 0.5),
-          showticklabels = TRUE
-        ),
-        yaxis = list(
-          tickfont = list(size = fontsize_row),
-          tickangle = row_text_angle,
-          tickvals = 1:nrow(x), ticktext = rownames(x),
-          linecolor = "#ffffff",
-          range = c(0.5, nrow(x) + 0.5),
-          showticklabels = TRUE
-        )
+    zmin = limits[1], zmax = limits[2]
+  ) %>%
+    layout(
+      xaxis = list(
+        tickfont = list(size = fontsize_col),
+        tickangle = column_text_angle,
+        tickvals = 1:ncol(x), ticktext = colnames(x),
+        linecolor = "#ffffff",
+        range = c(0.5, ncol(x) + 0.5),
+        showticklabels = TRUE
+      ),
+      yaxis = list(
+        tickfont = list(size = fontsize_row),
+        tickangle = row_text_angle,
+        tickvals = 1:nrow(x), ticktext = rownames(x),
+        linecolor = "#ffffff",
+        range = c(0.5, nrow(x) + 0.5),
+        showticklabels = TRUE
       )
-  p <- plotly::colorbar(p, lenmode = "fraction", title = key_title,
+    )
+  p <- plotly::colorbar(
+    p, lenmode = "fraction", title = key_title,
     xanchor = colorbar_xanchor, x = colorbar_xpos, y = colorbar_ypos,
-    yanchor = colorbar_yanchor, len = colorbar_len)
+    yanchor = colorbar_yanchor, len = colorbar_len
+  )
   p
 }
 
@@ -215,7 +313,7 @@ plotly_dend <- function(dend, side = c("row", "col"), flip = FALSE) {
 
   axis1 <- list(
     title = "",
-    range = c(0,  max(segs$y)),
+    range = c(0, max(segs$y)),
     linecolor = "#ffffff",
     showgrid = FALSE
   )
@@ -229,32 +327,34 @@ plotly_dend <- function(dend, side = c("row", "col"), flip = FALSE) {
   ## Have to change x and y depending on which orientation
   if (side == "row") {
     add_plot_lines <- function(p) {
-      p %>% add_segments(
-        x = ~y, xend = ~yend, y = ~x, yend = ~xend, color = ~col,
-        showlegend = FALSE,
-        colors = colors,
-        hoverinfo = "x"
+      p %>%
+        add_segments(
+          x = ~y, xend = ~yend, y = ~x, yend = ~xend, color = ~col,
+          showlegend = FALSE,
+          colors = colors,
+          hoverinfo = "x"
         ) %>%
-      layout(
-        hovermode = "closest",
-        xaxis = axis1,
-        yaxis = axis2
-      )
+        layout(
+          hovermode = "closest",
+          xaxis = axis1,
+          yaxis = axis2
+        )
     }
   }
   else {
     add_plot_lines <- function(p) {
-      p %>% add_segments(
-        x = ~x, xend = ~xend, y = ~y, yend = ~yend, color = ~col,
-        showlegend = FALSE,
-        colors = colors,
-        hoverinfo = "y"
-      ) %>%
-      layout(
-        hovermode = "closest",
-        xaxis = axis2,
-        yaxis = axis1
-      )
+      p %>%
+        add_segments(
+          x = ~x, xend = ~xend, y = ~y, yend = ~yend, color = ~col,
+          showlegend = FALSE,
+          colors = colors,
+          hoverinfo = "y"
+        ) %>%
+        layout(
+          hovermode = "closest",
+          xaxis = axis2,
+          yaxis = axis1
+        )
     }
   }
 
@@ -284,10 +384,9 @@ plotly_dend <- function(dend, side = c("row", "col"), flip = FALSE) {
 #' @return A ggplot geom_tile object
 #'
 ggplot_side_color_plot <- function(df, palette = NULL,
-    scale_title = paste(type, "side colors"), type = c("column", "row"),
-    text_angle = if (type == "column") 0 else 90, is_colors = FALSE, fontsize,
-    label_name = NULL) {
-
+                                   scale_title = paste(type, "side colors"), type = c("column", "row"),
+                                   text_angle = if (type == "column") 0 else 90, is_colors = FALSE, fontsize,
+                                   label_name = NULL) {
   type <- match.arg(type)
   if (is.matrix(df)) df <- as.data.frame(df)
   assert_that(is.data.frame(df))
@@ -304,10 +403,11 @@ ggplot_side_color_plot <- function(df, palette = NULL,
 
   ## Custom label
   if (!missing(label_name)) type <- label_name
-  if (type %in% colnames(df))
+  if (type %in% colnames(df)) {
     stop("Having", type, "in the colnames of the side_color df will drop data!")
+  }
 
-  df[[type]] <- if(!is.null(rownames(df))) rownames(df) else 1:nrow(df)
+  df[[type]] <- if (!is.null(rownames(df))) rownames(df) else 1:nrow(df)
 
   df[[type]] <- factor(df[[type]], levels = df[[type]], ordered = TRUE)
   df <- reshape2::melt(df, id.vars = type)
@@ -317,23 +417,23 @@ ggplot_side_color_plot <- function(df, palette = NULL,
 
   common_theme <- theme(
     panel.background = element_blank(),
-    axis.ticks = element_blank())
+    axis.ticks = element_blank()
+  )
 
   ## Don't need this hack any more?
   # if(original_dim[2] > 1) {
-    text_element <- element_text(angle = text_angle)
+  text_element <- element_text(angle = text_angle)
   # } else text_element <- element_blank()
 
   if (type == "column") {
-    mapping <- aes_string(x = id_var, y = "variable", fill = "value")
+    mapping <- aes_string(x = paste_aes(id_var), y = "variable", fill = "value")
 
     specific_theme <- theme(
       axis.text.x = element_blank(),
       axis.text.y = text_element
     )
   } else {
-
-    mapping <- aes_string(x = "variable", y = id_var, fill = "value")
+    mapping <- aes_string(x = "variable", y = paste_aes(id_var), fill = "value")
     specific_theme <- theme(
       axis.text.x = text_element,
       axis.text.y = element_blank()
@@ -344,8 +444,10 @@ ggplot_side_color_plot <- function(df, palette = NULL,
   if (is.function(palette)) {
     palette <- setNames(palette(nlevels(df[["value"]])), levels(df[["value"]]))
   } else if (!all(levels(factor(df[["value"]])) %in% names(palette))) {
-    stop(paste0("Not all levels of the ", type,
-      "_side_colors are mapped in the ", type, "_side_palette"))
+    stop(paste0(
+      "Not all levels of the ", type,
+      "_side_colors are mapped in the ", type, "_side_palette"
+    ))
   }
 
   g <- ggplot(df, mapping = mapping) +
@@ -355,7 +457,8 @@ ggplot_side_color_plot <- function(df, palette = NULL,
     scale_fill_manual(
       name = NULL,
       breaks = levels(df[["value"]]),
-      values = palette[levels(df[["value"]])]) +
+      values = palette[levels(df[["value"]])]
+    ) +
     theme
   return(g)
 }
@@ -368,12 +471,16 @@ default_side_colors <- function(n) {
     if (n <= 12) {
       RColorBrewer::brewer.pal(n, "Paired")[seq_len(n)]
     } else if (n <= 20) {
-      c(RColorBrewer::brewer.pal(12, "Paired"),
-        RColorBrewer::brewer.pal(n - 12, "Set2"))[seq_len(n)]
+      c(
+        RColorBrewer::brewer.pal(12, "Paired"),
+        RColorBrewer::brewer.pal(n - 12, "Set2")
+      )[seq_len(n)]
     } else if (n <= 32) {
-      c(RColorBrewer::brewer.pal(12, "Paired"),
+      c(
+        RColorBrewer::brewer.pal(12, "Paired"),
         RColorBrewer::brewer.pal(8, "Set2"),
-        RColorBrewer::brewer.pal(n - 20, "Set3"))[seq_len(n)]
+        RColorBrewer::brewer.pal(n - 20, "Set3")
+      )[seq_len(n)]
     } else {
       colorspace::rainbow_hcl(n)
     }
@@ -383,8 +490,7 @@ default_side_colors <- function(n) {
 ## Predict luminosity of cells and change text based on that
 ## http://stackoverflow.com/questions/12043187/how-to-check-if-hex-color-is-too-black
 predict_colors <- function(p, colorscale_df=p$x$data[[1]]$colorscale,
-    cell_values=p$x$data[[1]]$z, plot_method=c("ggplot", "plotly")) {
-
+                           cell_values=p$x$data[[1]]$z, plot_method=c("ggplot", "plotly")) {
   plot_method <- match.arg(plot_method)
 
   cell_values <- as.data.frame(cell_values)
@@ -407,7 +513,8 @@ predict_colors <- function(p, colorscale_df=p$x$data[[1]]$colorscale,
 
     ## Then need to sort, find nearest neighbour, and map across
     cell_values_vector_sort <- sort(cell_values_vector)
-    nearest_neighbours <- sapply(cell_values_vector_sort,
+    nearest_neighbours <- sapply(
+      cell_values_vector_sort,
       function(val) {
         max(colorscale_df[as.numeric(colorscale_df[, 1]) <= val, 1])
       }
@@ -422,6 +529,7 @@ predict_colors <- function(p, colorscale_df=p$x$data[[1]]$colorscale,
 
   # apply colors only to non-NA cells
   # In the future, it might be worth using na.color
+  cell_colors <- as.character(cell_colors)
   cell_colors[is.na(cell_colors)] <- "#ffffff" # make the default white for NA values
 
   cell_colors_rgb <- colorspace::hex2RGB(cell_colors)
@@ -432,7 +540,7 @@ predict_colors <- function(p, colorscale_df=p$x$data[[1]]$colorscale,
       luma <- (0.2126 * col[1]) +
         (0.7152 * col[2]) +
         (0.0722 * col[3])
-      ifelse (luma < 0.4, "white", "black")
+      ifelse(luma < 0.4, "white", "black")
     }
   )
   cell_font_colors
@@ -440,12 +548,18 @@ predict_colors <- function(p, colorscale_df=p$x$data[[1]]$colorscale,
 
 
 parse_plotly_color <- function(color) {
-  r <- gsub("rgb[a]?\\((\\d+),(\\d+),(\\d+)(,\\d+)?)",
-    "\\1", color)
-  g <- gsub("rgb[a]?\\((\\d+),(\\d+),(\\d+)(,\\d+)?)",
-    "\\2", color)
-  b <- gsub("rgb[a]?\\((\\d+),(\\d+),(\\d+)(,\\d+)?)",
-    "\\3", color)
+  r <- gsub(
+    "rgb[a]?\\((\\d+),(\\d+),(\\d+)(,\\d+)?)",
+    "\\1", color
+  )
+  g <- gsub(
+    "rgb[a]?\\((\\d+),(\\d+),(\\d+)(,\\d+)?)",
+    "\\2", color
+  )
+  b <- gsub(
+    "rgb[a]?\\((\\d+),(\\d+),(\\d+)(,\\d+)?)",
+    "\\3", color
+  )
   rgb(r, g, b, maxColorValue = 255)
 }
 
@@ -515,31 +629,30 @@ k_colors <- function(k) {
 
 # Create a plotly colorscale from a list of colors in any format.
 discrete_colorscale <- function(colors) {
-    colors <- rep(colors, each=100)
-    seq <- seq(0, 1, length.out = length(colors))
-    setNames(data.frame(seq, colors), NULL)
+  colors <- rep(colors, each = 100)
+  seq <- seq(0, 1, length.out = length(colors))
+  setNames(data.frame(seq, colors), NULL)
 }
 
 #' @importFrom stats setNames
 plotly_side_color_plot <- function(df, palette = NULL,
-    scale_title = paste(type, "side colors"), type = c("column", "row"),
-    text_angle = if (type == "column") 0 else 90, is_colors = FALSE,
-    label_name = NULL, fontsize = 10) {
-
+                                   scale_title = paste(type, "side colors"), type = c("column", "row"),
+                                   text_angle = if (type == "column") 0 else 90, is_colors = FALSE,
+                                   label_name = NULL, fontsize = 10) {
   type <- match.arg(type)
 
   if (is.null(label_name)) label_name <- type
 
   data <- df
-  data[] <- lapply(df, as.character)
-  if (type == "column") {
-      data <- t(data)
-  }
-  data <- as.data.frame(data)
+  data[] <- lapply(df, factor)
+  # if (type == "column") {
+  #   data <- t(data)
+  # }
+  data <- as.data.frame(data, stringsAsFactors = TRUE)
+  data[] <- lapply(data, factor)
   data_vals <- unlist(data)
-
-  levels <- sort(unique(data_vals))
-  levels <- setdiff(levels, NA)
+  levels <- levels(data_vals)
+  levels <- levels[!is.na(levels)]
 
   if (is.null(palette)) palette <- default_side_colors
 
@@ -548,7 +661,8 @@ plotly_side_color_plot <- function(df, palette = NULL,
   } else if (!all(levels %in% names(palette))) {
     stop(paste0(
       "Not all levels of the ", type,
-      "_side_colors are mapped in the ", type, "_side_palette"))
+      "_side_colors are mapped in the ", type, "_side_palette"
+    ))
   }
 
   levs2colors <- palette[as.character(levels)]
@@ -557,11 +671,14 @@ plotly_side_color_plot <- function(df, palette = NULL,
   df_nums <- data
   df_nums[] <- lapply(data, function(col) as.numeric(levs2nums[as.character(col)]))
   df_nums <- as.matrix(df_nums)
-
+  if (type == "column") {
+    df_nums <- t(df_nums)
+  }
   key_title <- paste(type, "annotation")
 
   text_mat <- data
-  text_mat[] <- lapply(seq_along(text_mat),
+  text_mat[] <- lapply(
+    seq_along(text_mat),
     function(i) {
       if (type == "row") {
         paste0(
@@ -579,26 +696,36 @@ plotly_side_color_plot <- function(df, palette = NULL,
     }
   )
 
+  ## Ensure tickvals are in right position when n = 2
+  offset <- (1 / length(levels)) / 2
   ## https://stackoverflow.com/questions/42524450/using-discrete-custom-color-in-a-plotly-heatmap
-  p <- plot_ly(z = df_nums, x = 1:ncol(df_nums), y = 1:nrow(df_nums),
+  p <- plot_ly(
+    z = df_nums, x = 1:ncol(df_nums), y = 1:nrow(df_nums),
     text = as.matrix(text_mat), hoverinfo = "text",
     type = "heatmap", showlegend = FALSE, colors = levs2colors,
     colorscale = discrete_colorscale(levs2colors),
     colorbar = list(
       # Capitalise first letter
       title = paste(gsub("^(\\w)", "\\U\\1", type, perl = TRUE), "annotation"),
-      tickmode = 'array',
-      tickvals = seq(1.5, length(levels) - 0.5, length.out = length(levels)),
+      tickmode = "array",
+      ## Issue #137
+      tickvals = seq(
+        1 + offset,
+        length(levels) - offset,
+        length.out = length(levels)
+      ),
       ticktext = levels,
-      len = 0.2))
+      len = 0.2
+    )
+  )
   if (type == "row") {
     p <- p %>% layout(
       xaxis = list(
         tickfont = list(size = fontsize),
         tickangle = text_angle,
-        tickvals = 1:ncol(data), ticktext = colnames(data),
+        tickvals = 1:ncol(df_nums), ticktext = colnames(df_nums),
         linecolor = "#ffffff",
-        range = c(0.5, ncol(data) + 0.5),
+        range = c(0.5, ncol(df_nums) + 0.5),
         showticklabels = TRUE
       ),
       yaxis = list(showticklabels = FALSE)
@@ -608,9 +735,9 @@ plotly_side_color_plot <- function(df, palette = NULL,
       yaxis = list(
         tickfont = list(size = fontsize),
         tickangle = text_angle,
-        tickvals = 1:nrow(data), ticktext = rownames(data),
+        tickvals = 1:nrow(df_nums), ticktext = rownames(df_nums),
         linecolor = "#ffffff",
-        range = c(0.5, nrow(data) + 0.5),
+        range = c(0.5, nrow(df_nums) + 0.5),
         showticklabels = TRUE
       ),
       xaxis = list(showticklabels = FALSE)
@@ -623,7 +750,6 @@ plotly_side_color_plot <- function(df, palette = NULL,
 
 
 
-
 #' @import webshot
 NULL
 # just so to have an excuse for why webshot is in import (the real reason is that plotly has it as suggests while it is used there by plotly::export)
@@ -641,36 +767,38 @@ hmly_to_file_1file <- function(hmly, file, width = NULL, height = NULL, ...) {
   # tools::file_ext("hithere.html")
   # tools::file_ext("hithere.html.png")
   file_extension <- tolower(tools::file_ext(file))
-  if(!(file_extension %in% c("html","pdf", "png", "jpeg"))) {
+  if (!(file_extension %in% c("html", "pdf", "png", "jpeg"))) {
     stop(paste(
       "file extension must be one of the following:",
       "\"html\", \"pdf\", \"png\", or \"jpeg\".",
-      "Since it was not, your heatmaply was not saved to a file."))
+      "Since it was not, your heatmaply was not saved to a file."
+    ))
   } else {
-    if(file_extension == "html") {
+    if (file_extension == "html") {
       # solution to dealing with the folder:
       # https://stackoverflow.com/questions/41399795/savewidget-from-htmlwidget-in-r-cannot-save-html-file-in-another-folder
       tmp_fp <- file
       tmp_fp <- file.path(normalizePath(dirname(tmp_fp)), basename(tmp_fp))
       hmly %>% htmlwidgets::saveWidget(file = tmp_fp, selfcontained = TRUE)
     }
-    if(file_extension %in% c("pdf", "png", "jpeg")) {
+    if (file_extension %in% c("pdf", "png", "jpeg")) {
       if (is.null(width)) {
         width <- size_default(file_extension, "width")
       }
       if (is.null(height)) {
         height <- size_default(file_extension, "height")
       }
-      plotly::export(hmly,
+      plotly::export(
+        hmly,
         file = file,
         vwidth = width,
         vheight = height,
-        cliprect="viewport")
+        cliprect = "viewport"
+      )
 
-      if(file_extension == "pdf") {
+      if (file_extension == "pdf") {
         warning("Due to a bug in the webshot package (which is used by plotly and heatmaply to create the pdf), the pdf is created with a white space around the plot. Until this bug is resolved you can manually trim the pdf using online tools such as: https://www.sejda.com/crop-pdf")
       }
-
     }
   }
   invisible(hmly)
@@ -694,5 +822,3 @@ size_default <- function(file_extension, direction=c("width", "height")) {
 bitmap_types <- c("png", "jpeg")
 
 hmly_to_file <- Vectorize(hmly_to_file_1file, vectorize.args = "file")
-
-# hmly_to_file(p, c("hm.png", "hm.html"))
