@@ -92,37 +92,38 @@ NULL
 # # menus are added and removed as needed: !!
 #
 
-.onAttach <- function(lib, pkg,...){
-   packageStartupMessage(heatmaplyWelcomeMessage())
+.onAttach <- function(lib, pkg, ...) {
+  packageStartupMessage(heatmaplyWelcomeMessage())
 }
 
 #
 #
 
-heatmaplyWelcomeMessage <- function(){
-   # library(utils)
 
-   paste0("\n",
-         "======================\n",
-         "Welcome to heatmaply version ", utils::packageDescription("heatmaply")$Version, "\n",
-         # "\n",
-         "\n",
-         "Type citation('heatmaply') for how to cite the package.\n",
-         "Type ?heatmaply for the main documentation.\n",
-         "\n",
-         "The github page is: https://github.com/talgalili/heatmaply/\n",
-         "Please submit your suggestions and bug-reports at: https://github.com/talgalili/heatmaply/issues\n",
-         "Or contact: <tal.galili@gmail.com>\n",
-         # "\n",
-         # "\tTo suppress this message use:  ", "suppressPackageStartupMessages(library(heatmaply))\n",
-         "======================\n"
-   )
+heatmaplyWelcomeMessage <- function() {
+  # library(utils)
+
+  paste0(
+    "\n",
+    "======================\n",
+    "Welcome to heatmaply version ", utils::packageDescription("heatmaply")$Version, "\n",
+    # "\n",
+    "\n",
+    "Type citation('heatmaply') for how to cite the package.\n",
+    "Type ?heatmaply for the main documentation.\n",
+    "\n",
+    "The github page is: https://github.com/talgalili/heatmaply/\n",
+    "Please submit your suggestions and bug-reports at: https://github.com/talgalili/heatmaply/issues\n",
+    "Or contact: <tal.galili@gmail.com>\n",
+    # "\n",
+    # "\tTo suppress this message use:  ", "suppressPackageStartupMessages(library(heatmaply))\n",
+    "======================\n"
+  )
 }
 
 
 
 
-
 # using "zzz.r" like in devtools...
 
 # When adding new files, make sure they are listed in DESCRIPTION:
@@ -267,6 +268,13 @@ heatmaplyWelcomeMessage <- function(){
 
 
 
+# https://github.com/r-lib/styler
+# # install.packages("remotes")
+# remotes::install_github("krlmlr/styler")
+# styler::style_pkg()
+
+# devtools::spell_check()
+
 
 
 
@@ -276,6 +284,7 @@ heatmaplyWelcomeMessage <- function(){
 
 
 
+
 # when a function is renamed, its document in man must be removed - otherwise it may cause problems with the built check (it will try to run the code in the example, and will fail.)
 # When all is done, run:
 # library(devtools)
@@ -296,5 +305,3 @@ heatmaplyWelcomeMessage <- function(){
 # devtools::build_win(version="R-release")
 # devtools::build_win(version="R-devel")
 # release()
-
-
